package com.compomics.util.parameters.identification.advanced;

import com.compomics.util.experiment.personalization.ExperimentObject;
import com.compomics.util.experiment.identification.modification.ModificationLocalizationScore;

/**
 * This class contains the modification localization preferences.
 *
 * @author Marc Vaudel
 * @author Harald Barsnes
 */
public class ModificationLocalizationParameters extends ExperimentObject {

    /**
     * Boolean indicating whether a probabilistic score is to be calculated.
     */
    private boolean probabilisticScoreCalculation = true;
    /**
     * The probabilistic score selected.
     */
    private ModificationLocalizationScore selectedProbabilisticScore = ModificationLocalizationScore.PhosphoRS;
    /**
     * The D-score threshold.
     */
    private double dScoreThreshold = 95;
    /**
     * The probabilistic score threshold.
     */
    private double probabilisticScoreThreshold = 95;
    /**
     * Boolean indicating whether neutral losses shall be accounted for in the
     * calculation of the probabilistic score.
     */
    private boolean probabilisticScoreNeutralLosses = false;
    /**
     * The preferences to use when matching modifications to amino acid
     * sequences.
     */
    private SequenceMatchingParameters sequenceMatchingParameters;
    /**
     * Boolean indicating whether the non confidently localized modification
     * should be aligned on the confident sites.
     */
    private boolean alignNonConfidentModifications = true;

    /**
     * Constructor.
     */
    public ModificationLocalizationParameters() {

        sequenceMatchingParameters = new SequenceMatchingParameters();
        sequenceMatchingParameters.setSequenceMatchingType(SequenceMatchingParameters.MatchingType.aminoAcid);

    }

    /**
     * Indicates whether a probabilistic modification score is required.
     *
     * @return a boolean indicating whether a probabilistic modification score
     * is required
     */
    public boolean isProbabilisticScoreCalculation() {
        

        return probabilisticScoreCalculation;

    }

    /**
     * Sets whether a probabilistic modification score is required.
     *
     * @param probabilisticScoreCalculation a boolean indicating whether a
     * probabilistic modification score is required
     */
    public void setProbabilisticScoreCalculation(boolean probabilisticScoreCalculation) {
        
        this.probabilisticScoreCalculation = probabilisticScoreCalculation;

    }

    /**
     * Returns the selected probabilistic score.
     *
     * @return the selected probabilistic score
     */
    public ModificationLocalizationScore getSelectedProbabilisticScore() {
        

        return selectedProbabilisticScore;

    }

    /**
     * Sets the selected probabilistic score.
     *
     * @param selectedProbabilisticScore the selected probabilistic score
     */
    public void setSelectedProbabilisticScore(ModificationLocalizationScore selectedProbabilisticScore) {

        
        this.selectedProbabilisticScore = selectedProbabilisticScore;

    }

    /**
     * Returns the probabilistic score threshold (inclusive).
     *
     * @return The probabilistic score threshold (inclusive).
     */
    public double getProbabilisticScoreThreshold() {
        

        return probabilisticScoreThreshold;

    }

    /**
     * Sets the probabilistic score threshold (inclusive).
     *
     * @param probabilisticScoreThreshold The probabilistic score threshold
     * (inclusive).
     */
    public void setProbabilisticScoreThreshold(
            double probabilisticScoreThreshold
    ) {

        
        this.probabilisticScoreThreshold = probabilisticScoreThreshold;

    }

    /**
     * Returns the D-score threshold.
     *
     * @return the D-score threshold
     */
    public double getDScoreThreshold() {
        

        return dScoreThreshold;

    }

    /**
     * Sets the D-score threshold.
     *
     * @param dScoreThreshold the D-score threshold
     */
    public void setDScoreThreshold(double dScoreThreshold) {

        
        this.dScoreThreshold = dScoreThreshold;

    }

    /**
     * Indicates whether the neutral losses shall be taken into account for
     * spectrum annotation when calculating the probabilistic score.
     *
     * @return a boolean indicating whether the neutral losses shall be taken
     * into account for spectrum annotation when calculating the probabilistic
     * score
     */
    public boolean isProbabilisticScoreNeutralLosses() {
        

        return probabilisticScoreNeutralLosses;

    }

    /**
     * Sets whether the neutral losses shall be taken into account for spectrum
     * annotation when calculating the probabilistic score.
     *
     * @param probabilisticScoreNeutralLosses indicates whether the neutral
     * losses shall be taken into account for spectrum annotation when
     * calculating the probabilistic score
     */
    public void setProbabilisticScoreNeutralLosses(boolean probabilisticScoreNeutralLosses) {

        
        this.probabilisticScoreNeutralLosses = probabilisticScoreNeutralLosses;

    }

    /**
     * Returns the sequence matching preferences to use when mapping
     * modifications on amino acid sequences.
     *
     * @return the sequence matching preferences to use when mapping
     * modifications on amino acid sequences
     */
    public SequenceMatchingParameters getSequenceMatchingParameters() {
        

        return sequenceMatchingParameters;

    }

    /**
     * Sets the sequence matching preferences to use when mapping modifications
     * on amino acid sequences.
     *
     * @param sequenceMatchingParameters the sequence matching preferences to
     * use when mapping modifications on amino acid sequences
     */
    public void setSequenceMatchingParameters(SequenceMatchingParameters sequenceMatchingParameters) {

        
        this.sequenceMatchingParameters = sequenceMatchingParameters;

    }

    /**
     * Indicates whether the non confidently localized modifications should be
     * aligned on the confident sites.
     *
     * @return boolean indicating whether the non confidently localized
     * modifications should be aligned on the confident sites
     */
    public boolean getAlignNonConfidentModifications() {
        

        return alignNonConfidentModifications;

    }

    /**
     * Sets whether the non confidently localized modifications should be
     * aligned on the confident sites.
     *
     * @param alignNonConfidentModifications a boolean indicating whether the
     * non confidently localized modifications should be aligned on the
     * confident sites
     */
    public void setAlignNonConfidentModifications(boolean alignNonConfidentModifications) {

        
        this.alignNonConfidentModifications = alignNonConfidentModifications;

    }

    /**
     * Returns a short description of the parameters.
     *
     * @return a short description of the parameters
     */
    public String getShortDescription() {
        

        String newLine = System.getProperty("line.separator");

        StringBuilder output = new StringBuilder();

        output.append("Score: ").append(selectedProbabilisticScore).append(".").append(newLine);
        output.append("Include Neutral Losses: ").append(probabilisticScoreNeutralLosses).append(".").append(newLine);
        output.append("Threshold: ").append(probabilisticScoreThreshold).append(".").append(newLine);
        output.append("Align modifications: ").append(getAlignNonConfidentModifications()).append(".").append(newLine);

        return output.toString();

    }

    /**
     * Returns true if the objects have identical settings.
     *
     * @param otherParameters the parameters to compare to
     *
     * @return true if the objects have identical settings
     */
    public boolean equals(ModificationLocalizationParameters otherParameters) {
        

        if (otherParameters == null) {

            return false;

        }

        if (probabilisticScoreCalculation != otherParameters.isProbabilisticScoreCalculation()) {

            return false;

        }

        if (selectedProbabilisticScore != otherParameters.getSelectedProbabilisticScore()) {

            return false;

        }

        if (probabilisticScoreNeutralLosses != otherParameters.isProbabilisticScoreNeutralLosses()) {

            return false;

        }

        if (!getAlignNonConfidentModifications() == otherParameters.getAlignNonConfidentModifications()) {

            return false;

        }

        if (!sequenceMatchingParameters.isSameAs(otherParameters.getSequenceMatchingParameters())) {

            return false;

        }

        return true;

    }
}
