package com.compomics.util.gui.protein;

import com.compomics.util.experiment.io.biology.protein.FastaParameters;
import com.compomics.util.gui.error_handlers.HelpDialog;
import com.compomics.util.gui.renderers.AlignedListCellRenderer;
import java.awt.Dialog;
import java.awt.Toolkit;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;

/**
 * A dialog for advanced database settings.
 *
 * @author Marc Vaudel
 * @author Harald Barsnes
 */
public class FastaParametersDialog extends javax.swing.JDialog {

    /**
     * The parent frame.
     */
    private java.awt.Frame parentFrame;
    /**
     * Boolean indicating whether the user canceled the editing.
     */
    private boolean canceled = false;
    /**
     * Boolean indicating whether the settings can be edited by the user.
     */
    private boolean editable;

    /**
     * Creates a new FractionSettingsDialog with a frame as owner.
     *
     * @param parentFrame a parent frame
     * @param fastaSettings the FASTA settings
     * @param editable boolean indicating whether the settings can be edited by
     * the user
     */
    public FastaParametersDialog(java.awt.Frame parentFrame, FastaParameters fastaSettings, boolean editable) {
        super(parentFrame, true);
        this.parentFrame = parentFrame;
        this.editable = editable;
        initComponents();
        setUpGui(fastaSettings);
        setLocationRelativeTo(parentFrame);
        setVisible(true);
    }

    /**
     * Creates a new FractionSettingsDialog with a dialog as owner.
     *
     * @param owner the dialog owner
     * @param parentFrame a parent frame
     * @param fastaSettings the FASTA settings
     * @param editable boolean indicating whether the settings can be edited by
     * the user
     */
    public FastaParametersDialog(Dialog owner, java.awt.Frame parentFrame, FastaParameters fastaSettings, boolean editable) {
        super(owner, true);
        this.parentFrame = parentFrame;
        this.editable = editable;
        initComponents();
        setUpGui(fastaSettings);
        setLocationRelativeTo(owner);
        setVisible(true);
    }

    /**
     * Updates the text in the text field.
     */
    private void setUpGui(FastaParameters fastaSettings) {

        decoyTypeComboBox.setRenderer(new AlignedListCellRenderer(SwingConstants.CENTER));
        targetDecoyComboBox.setRenderer(new AlignedListCellRenderer(SwingConstants.CENTER));

        if (fastaSettings.isTargetDecoy()) {
            targetDecoyComboBox.setSelectedIndex(0);
        } else {
            targetDecoyComboBox.setSelectedIndex(1);
        }

        decoyFlagTxt.setText(fastaSettings.getDecoyFlag());

        if (fastaSettings.isDecoySuffix()) {
            decoyTypeComboBox.setSelectedIndex(1);
        } else {
            decoyTypeComboBox.setSelectedIndex(0);
        }

        fastaSuffixTxt.setText(fastaSettings.getTargetDecoyFileNameSuffix());

        targetDecoyComboBox.setEnabled(editable);
        decoyFlagTxt.setEnabled(editable);
        decoyTypeComboBox.setEnabled(editable);
        fastaSuffixTxt.setEnabled(editable);
    }

    /**
     * Indicates whether the user canceled the editing.
     *
     * @return a boolean indicating whether the user canceled the editing
     */
    public boolean isCanceled() {
        return canceled;
    }

    /**
     * Returns the FASTA settings as set by the user.
     *
     * @return the FASTA settings as set by the user
     */
    public FastaParameters getFastaSettings() {
        FastaParameters fastaSettings = new FastaParameters();
        fastaSettings.setTargetDecoy(targetDecoyComboBox.getSelectedIndex() == 0);
        fastaSettings.setDecoyFlag(decoyFlagTxt.getText());
        fastaSettings.setDecoySuffix(decoyTypeComboBox.getSelectedIndex() == 1);
        fastaSettings.setTargetDecoyFileNameSuffix(fastaSuffixTxt.getText());
        return fastaSettings;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        backgroundPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        fileProcessingPanel = new javax.swing.JPanel();
        fastaSuffixTxt = new javax.swing.JTextField();
        decoyTagLabel = new javax.swing.JLabel();
        decoyFlagTxt = new javax.swing.JTextField();
        decoySuffixLabel = new javax.swing.JLabel();
        decoyTypeLabel = new javax.swing.JLabel();
        decoyTypeComboBox = new javax.swing.JComboBox<>();
        targetDecoyLabel = new javax.swing.JLabel();
        targetDecoyComboBox = new javax.swing.JComboBox<>();
        helpJButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Database Processing");

        backgroundPanel.setBackground(new java.awt.Color(230, 230, 230));

        okButton.setText("OK");
        okButton.setPreferredSize(new java.awt.Dimension(75, 25));
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.setPreferredSize(new java.awt.Dimension(75, 25));
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        fileProcessingPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Settings"));
        fileProcessingPanel.setOpaque(false);

        fastaSuffixTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);

        decoyTagLabel.setText("Decoy Tag");

        decoyFlagTxt.setHorizontalAlignment(javax.swing.JTextField.CENTER);

        decoySuffixLabel.setText("Decoy File Tag");

        decoyTypeLabel.setText("Decoy Type");

        decoyTypeComboBox.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Prefix", "Suffix" }));
        decoyTypeComboBox.setSelectedIndex(1);

        targetDecoyLabel.setText("Target-Decoy");

        targetDecoyComboBox.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Yes", "No" }));
        targetDecoyComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targetDecoyComboBoxActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout fileProcessingPanelLayout = new javax.swing.GroupLayout(fileProcessingPanel);
        fileProcessingPanel.setLayout(fileProcessingPanelLayout);
        fileProcessingPanelLayout.setHorizontalGroup(
            fileProcessingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(fileProcessingPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(fileProcessingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(decoyTagLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(decoyTypeLabel)
                    .addComponent(decoySuffixLabel)
                    .addComponent(targetDecoyLabel))
                .addGap(18, 18, 18)
                .addGroup(fileProcessingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(targetDecoyComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(fastaSuffixTxt)
                    .addComponent(decoyFlagTxt)
                    .addComponent(decoyTypeComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        fileProcessingPanelLayout.setVerticalGroup(
            fileProcessingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(fileProcessingPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(fileProcessingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(targetDecoyLabel)
                    .addComponent(targetDecoyComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(fileProcessingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(decoyTagLabel)
                    .addComponent(decoyFlagTxt, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(fileProcessingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(decoyTypeLabel)
                    .addComponent(decoyTypeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(fileProcessingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(fastaSuffixTxt, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(decoySuffixLabel))
                .addContainerGap())
        );

        helpJButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/help.GIF"))); // NOI18N
        helpJButton.setToolTipText("Help");
        helpJButton.setBorder(null);
        helpJButton.setBorderPainted(false);
        helpJButton.setContentAreaFilled(false);
        helpJButton.setFocusable(false);
        helpJButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        helpJButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        helpJButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                helpJButtonMouseEntered(evt);
            }
            public void mouseExited(java.awt.event.MouseEvent evt) {
                helpJButtonMouseExited(evt);
            }
        });
        helpJButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpJButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout backgroundPanelLayout = new javax.swing.GroupLayout(backgroundPanel);
        backgroundPanel.setLayout(backgroundPanelLayout);
        backgroundPanelLayout.setHorizontalGroup(
            backgroundPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, backgroundPanelLayout.createSequentialGroup()
                .addGroup(backgroundPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(backgroundPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(fileProcessingPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(backgroundPanelLayout.createSequentialGroup()
                        .addGap(15, 15, 15)
                        .addComponent(helpJButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 297, Short.MAX_VALUE)
                        .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, 47, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );

        backgroundPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {cancelButton, okButton});

        backgroundPanelLayout.setVerticalGroup(
            backgroundPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(backgroundPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(fileProcessingPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(backgroundPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(helpJButton)
                    .addComponent(okButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(cancelButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(backgroundPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(backgroundPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Save the settings and close the dialog.
     *
     * @param evt
     */
    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed

        boolean valid = true;

        // check that decoy tag and decoy file tag are not empty
        if (decoyFlagTxt.getText().isEmpty()) {
            JOptionPane.showMessageDialog(this, "The decoy tag cannot be empty.", "Warning", JOptionPane.WARNING_MESSAGE);
            valid = false;
        }
        if (fastaSuffixTxt.getText().isEmpty()) {
            JOptionPane.showMessageDialog(this, "The decoy file tag cannot be empty.", "Warning", JOptionPane.WARNING_MESSAGE);
            valid = false;
        }

        if (valid) {
            dispose();
        }
    }//GEN-LAST:event_okButtonActionPerformed

    /**
     * Close the dialog without saving the settings.
     *
     * @param evt
     */
    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        canceled = true;
        dispose();
    }//GEN-LAST:event_cancelButtonActionPerformed

    /**
     * Change the cursor to a hand cursor.
     *
     * @param evt
     */
    private void helpJButtonMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_helpJButtonMouseEntered
        setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
    }//GEN-LAST:event_helpJButtonMouseEntered

    /**
     * Change the cursor back to the default cursor.
     *
     * @param evt
     */
    private void helpJButtonMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_helpJButtonMouseExited
        setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
    }//GEN-LAST:event_helpJButtonMouseExited

    /**
     * Open the FASTA processing help page.
     *
     * @param evt
     */
    private void helpJButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpJButtonActionPerformed
        setCursor(new java.awt.Cursor(java.awt.Cursor.WAIT_CURSOR));
        new HelpDialog(parentFrame, getClass().getResource("/helpFiles/FastaPreferences.html"),
                Toolkit.getDefaultToolkit().getImage(getClass().getResource("/icons/help.GIF")),
                Toolkit.getDefaultToolkit().getImage(getClass().getResource("/icons/peptide-shaker.gif")),
                "FASTA Processing - Help");
        setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
    }//GEN-LAST:event_helpJButtonActionPerformed

    /**
     * Enable or disable the decoy options.
     *
     * @param evt
     */
    private void targetDecoyComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targetDecoyComboBoxActionPerformed
        boolean targetDecoy = targetDecoyComboBox.getSelectedIndex() == 0;
        decoyFlagTxt.setEnabled(targetDecoy);
        decoyTypeComboBox.setEnabled(targetDecoy);
    }//GEN-LAST:event_targetDecoyComboBoxActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel backgroundPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JTextField decoyFlagTxt;
    private javax.swing.JLabel decoySuffixLabel;
    private javax.swing.JLabel decoyTagLabel;
    private javax.swing.JComboBox<String> decoyTypeComboBox;
    private javax.swing.JLabel decoyTypeLabel;
    private javax.swing.JTextField fastaSuffixTxt;
    private javax.swing.JPanel fileProcessingPanel;
    private javax.swing.JButton helpJButton;
    private javax.swing.JButton okButton;
    private javax.swing.JComboBox<String> targetDecoyComboBox;
    private javax.swing.JLabel targetDecoyLabel;
    // End of variables declaration//GEN-END:variables
}
